from django.db import models
from django.utils import timezone
from wagtail.admin.edit_handlers import (
    FieldPanel,
    MultiFieldPanel,
    PublishingPanel,
    StreamFieldPanel,
)
from wagtail.core.blocks import RichTextBlock
from wagtail.core.fields import StreamField
from wagtail.core.models import Page
from wagtail.images.edit_handlers import ImageChooserPanel

from shared.blocks import GalleryBlock, MenuItemBlock, MenuParentBlock


class SubpageMixin:
    """Must be used in class definition before MetadataPageMixin!"""

    @property
    def root_page(self):
        if not hasattr(self, "_root_page"):
            # vypada to hackove ale lze takto pouzit: dle dokumentace get_ancestors
            # vraci stranky v poradi od rootu, tedy domovska stranka je druha v poradi
            self._root_page = self.get_ancestors().specific()[1]
        return self._root_page

    def get_meta_image(self):
        return self.search_image or self.root_page.get_meta_image()


class ArticleMixin(models.Model):
    """
    Common fields for articles.

    Must be used in class definition before MetadataPageMixin!

    If you want to tag articles, add tags as `tags` field in article page model.
    """

    RICH_TEXT_FEATURES = [
        "h2",
        "h3",
        "h4",
        "bold",
        "italic",
        "ol",
        "embed",
        "ul",
        "link",
        "document-link",
        "image",
    ]

    ### FIELDS

    content = StreamField(
        [
            (
                "text",
                RichTextBlock(label="Textový editor", features=RICH_TEXT_FEATURES),
            ),
            ("gallery", GalleryBlock(label="Galerie")),
        ],
        verbose_name="Článek",
        blank=True,
    )
    date = models.DateField("datum", default=timezone.now)
    perex = models.TextField("perex")
    author = models.CharField("autor", max_length=250, blank=True, null=True)
    image = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        verbose_name="obrázek",
    )

    ### PANELS

    content_panels = Page.content_panels + [
        FieldPanel("date"),
        FieldPanel("perex"),
        StreamFieldPanel("content"),
        FieldPanel("author"),
        ImageChooserPanel("image"),
    ]

    settings_panels = [PublishingPanel()]

    class Meta:
        abstract = True

    @classmethod
    def has_tags(cls):
        try:
            cls._meta.get_field("tags")
        except models.FieldDoesNotExist:
            return False
        return True

    def tag_filter_page(self):
        """Page used for filtering by tags in url like `?tag=foo`."""
        return self.get_parent()

    def get_meta_image(self):
        return self.image

    def get_meta_description(self):
        if self.search_description:
            return self.search_description
        if len(self.perex) > 150:
            return str(self.perex)[:150] + "..."
        return self.perex


class MenuMixin(Page):
    menu = StreamField(
        [("menu_item", MenuItemBlock()), ("menu_parent", MenuParentBlock())],
        verbose_name="Menu",
        blank=True,
    )

    menu_panels = [
        MultiFieldPanel(
            [
                StreamFieldPanel("menu"),
            ],
            heading="Menu Options",
        ),
    ]

    class Meta:
        abstract = True
